//+------------------------------------------------------------------+
//|                                               HULL_PRECISION.mq4 |
//|                                                              TTK |
//|                                     https://youtu.be/LjIAzKo62MQ |
//+------------------------------------------------------------------+
#property copyright "TTK"
#property link      "https://youtu.be/LjIAzKo62MQ"
#property version   "1.00"
#property strict
#property indicator_chart_window
#property indicator_buffers 2

//---
#include <CUSTOM/VTLS.mqh>
//---

//---
extern string HULL_PRECISION_SETTINGS;
extern int zone_candle_start = 0;                              //ENTER CANDLE TO DISPLAY ZONES FROM
input color buy_zone_arrow_colour = clrMediumSeaGreen;         //ENTER BUY ZONE ARROW COLOUR
input color sell_zone_arrow_colour = clrRed;                   //ENTER SELL ZONE ARROW COLOUR
//---

//---
int candle_before_current = 1;
int buy_zone_index = 0;
int sell_zone_index = 0;
//---

//---
double BUY_ZONE[];
double SELL_ZONE[];
//---

//---
double before_previous_hull_ma;
double previous_hull_ma;
double current_hull_ma;
double trend_indication;
//---

//---
double buy_zone_candle_start_offset;
double sell_zone_candle_start_offset;
//---

//---
bool buy_zone_entered = FALSE;
bool sell_zone_entered = FALSE;
//---

//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//--- destroy timer
//EventKillTimer();

   if(UninitializeReason() == REASON_PROGRAM || UninitializeReason() == REASON_CHARTCHANGE || UninitializeReason() == REASON_CHARTCLOSE || UninitializeReason() == REASON_ACCOUNT || UninitializeReason() == REASON_CLOSE)
     {
      return;
     }
   else
     {
      for(int i = ObjectsTotal(); i >= 0; i--)
        {
         if(StringSubstr(ObjectName(i), 0, 18) == "BUY_ZONE_ACTIVATED" ||
            StringSubstr(ObjectName(i), 0, 19) == "SELL_ZONE_ACTIVATED")
           {
            ObjectDelete(ObjectName(i));
           }
        }
     }

   Comment("");
  }
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
   if(zone_candle_start <= 0)
     {
      zone_candle_start = Bars - 4;
      //zone_candle_start = Bars - 1;
     }

   if(Digits == 5)
     {
      buy_zone_candle_start_offset = 0.00300;
      sell_zone_candle_start_offset = 0.00300;
     }
   else
      if(Digits == 3)
        {
         buy_zone_candle_start_offset = 0.300;
         sell_zone_candle_start_offset = 0.300;
        }

   IndicatorDigits(Digits);
//--- indicator buffers mapping
   SetIndexStyle(0, DRAW_ARROW, STYLE_SOLID, 3, buy_zone_arrow_colour);
   SetIndexBuffer(0, BUY_ZONE);
   SetIndexLabel(0, "BUY_ZONE");
   SetIndexArrow(0, 233);
//---
   SetIndexStyle(1, DRAW_ARROW, STYLE_SOLID, 3, sell_zone_arrow_colour);
   SetIndexBuffer(1, SELL_ZONE);
   SetIndexLabel(1, "SELL_ZONE");
   SetIndexArrow(1, 234);
//---
   SetIndexLabel(0, "Buy Zone");
   SetIndexLabel(1, "Sell Zone");
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---
   if(is_new_candle())
     {
      for(int candle_number = MathMax(zone_candle_start - IndicatorCounted(), 1); candle_number > 0; candle_number--)
        {
         before_previous_hull_ma = iCustom(
                                      NULL,
                                      0,
                                      "/CUSTOM/HULL",
                                      20,
                                      2.0,
                                      PRICE_CLOSE,
                                      0,
                                      candle_number + 3
                                   );

         previous_hull_ma = iCustom(
                               NULL,
                               0,
                               "/CUSTOM/HULL",
                               20,
                               2.0,
                               PRICE_CLOSE,
                               0,
                               candle_number + 2
                            );

         current_hull_ma = iCustom(
                              NULL,
                              0,
                              "/CUSTOM/HULL",
                              20,
                              2.0,
                              PRICE_CLOSE,
                              0,
                              candle_number + 1
                           );


         trend_indication = iCustom(
                               NULL,
                               0,
                               "/CUSTOM/HULL",
                               20,
                               2.0,
                               PRICE_CLOSE,
                               3,
                               candle_number + 1
                            );


         if((before_previous_hull_ma - previous_hull_ma) > 0)
           {
            if((previous_hull_ma - before_previous_hull_ma) < 0)
              {
               if((previous_hull_ma - current_hull_ma) < 0)
                 {
                  if((current_hull_ma - previous_hull_ma) > 0)
                    {
                     if(trend_indication == 1 && !buy_zone_entered)
                       {
                        sell_zone_entered = FALSE;

                        BUY_ZONE[candle_number] = Low[candle_number] - buy_zone_candle_start_offset;

                        buy_zone_entered = TRUE;
                       }
                    }
                 }
              }
           }
         else
            if((before_previous_hull_ma - previous_hull_ma) < 0)
              {
               if((previous_hull_ma - before_previous_hull_ma) > 0)
                 {
                  if((previous_hull_ma - current_hull_ma) > 0)
                    {
                     if((current_hull_ma - previous_hull_ma) < 0)
                       {
                        if(trend_indication == -1 && !sell_zone_entered)
                          {
                           if(current_hull_ma < previous_hull_ma)
                             {
                              buy_zone_entered = FALSE;

                              SELL_ZONE[candle_number] = High[candle_number] + sell_zone_candle_start_offset;

                              sell_zone_entered = TRUE;
                             }
                          }
                       }
                    }
                 }
              }
        }
     }
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
//| Timer function                                                   |
//+------------------------------------------------------------------+
void OnTimer()
  {
//---

  }
//+------------------------------------------------------------------+
//| ChartEvent function                                              |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam)
  {
//---

  }
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
